// This file is part of the brownmove simulation package
// 
// V1.4  Aug 2012
// 
// (C) 2003 - 2012 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#ifndef PDBSHAPE_H
#define PDBSHAPE_H

#include <string>
#include <vector>
#include <iostream>
#include <string>
#include <sstream>
#include <stdio.h>

#include "types.h"
#include "constants.h"
#include "vectormath.h"

#include "shape.h"


namespace brown{
    /**
     *@author tg
     */
    
	//! PDBShape is only relevant for analysis: it allows to convert a CG protein into a (reduced) atomistic representation.
    class PDBShape : public Shape {
	
	public:
	PDBShape() { };
	PDBShape(const PDBShape &other);
	~PDBShape();
	
	
	/** restores the parts' (absolute and relative) positions from fullPosition */
	virtual void recalculate(const fullPosition &p);
	
	/** returns some string for testing */
	virtual std::string describe();
	
	//! adds another PDB atom with position and index to the vector 
	//! Also calculates the relative position of the atom from the molecules center given
	void addPDBAtom(PDBAtomPos pdbp, const position &moleculeCenter);
		
	//! returns the number of PDB atoms
	int size() { return(PDBAtoms.size()); }
	
	//! Dumps the known infos about PDB atoms in browndef format
	void dumpStructure();

	//! Count and dump the actual data and positions of the atoms to stringstream raus
	int dumpPDBAtoms(long int index, std::string *raus);

	
	private:
	//! The relative positions of the atoms and their data
	vector<PDBAtomPos> PDBAtoms;
	//! Keep track of the absolute positions of the atoms, updated when shape is moved
	vector<PDBAtomPos> actual_PDBAtoms;

    };
}

#endif
