// This file is part of the brownmove simulation package
// 
// V1.4  Aug 2012
// 
// (C) 2003 - 2012 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#ifndef SHAPE_H
#define SHAPE_H

#include <string>
#include <iostream>

#include "types.h"

using namespace std;

namespace brown{
    
    /**
	 *@author jni, tg
	 */
	
	//! Base class for the various shapes for all the numerous interactions.

class Shape {
    public:
	
	//! Constructor
    Shape();
	//! Desctructor
    ~Shape();
    
    
    //! restores the parts' positions from fullPosition
    virtual void recalculate(const fullPosition &p) = 0;
    
     
    //! returns some string for diagnostics
    // has to be implemented in every subclass
    virtual std::string describe() = 0;
    
    //! tell the shape of a wall where to find its wall's position and orientation
    void setPositionNormal(position *p, direction *n);
    
    //! Return the pointers to the wall's position and normal (for a wall's shape)
    position *getPosition() {
	return(myWallsPosition);
    }
    
    //! for testing purposes
    direction *getNormal() {
		return(myWallsNormal);
    }
    
    protected:
	//! Short-cut pointer to the wall's position (only for wall shapes)
    position *myWallsPosition;
	//! Short-cut pointer to wall's normal
    direction *myWallsNormal;
};
}

#endif
