// This file is part of the brownmove simulation package
// 
// V1.4  Aug 2012
// 
// (C) 2003 - 2012 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#ifndef STICKYSHAPE_H
#define STICKYSHAPE_H

#include <string>
#include <vector>
#include <iostream>
#include <string>
#include <sstream>

#include "types.h"
#include "constants.h"
#include "vectormath.h"

#include "shape.h"


namespace brown{
    /**
     *@author tg
     */
    
	//! StickyShape implements sticky patches in particles both in radial and angular direction
    class StickyShape : public Shape {
	
	public:
	StickyShape();
	StickyShape(const StickyShape &other);
	~StickyShape();
	
	
	/** restores the parts' (absolute and relative) positions from fullPosition */
	virtual void recalculate(const fullPosition &p);
	
	/** returns some string for testing */
	virtual std::string describe();
	
	//! adds another sticky patch sphere with position and radius to the vector 
	//! Also calculates the relatiev position of the sphere from the given molecule center
	void addPatch(stickyPos sp, const position &moleculeCenter);
	
	
	//! calculate the forces and torques from the patches of this and the other StickyShape
	void calculateImpact(fullForce &myForce, fullForce &otherForce, StickyShape *ss, const bool isPeriodic[3], const doublevec &cellSize);
	
	//! returns the number of charges
	int size() { return(stickies.size()); }
	
	//! Dumps the known infos about sticky patches in browndef format
	void dumpStructure();

	//! set the pointer to the array of parameter-structs for the sticky patches
	void setParameterArray(stickyParameter *theArray) {
		paramsArray = theArray;
	}

	//! Initialize the numeric ids of the individual sticky spheres, done when molecule is inserted into the cup.
	//! Returns the number of used ids.
	long int setID(const long int currNr);
	
	//! Reset the flags in the known-shape-memories of the spheres. 
	//! Called from Cup::tellMoleculesToMove()  => Gestalt::makeBrownianStep()
	void resetFlags();
	
	
	private:
	//! three vectors for a cordinate system of the sticky shape
	direction ex, ey, ez;
	//! The relative positions of the sticky patches
	vector<stickyPos> stickies;
	//! Keep track of the absolute positions of the patches, updated when shape is moved
	vector<stickyPos> actual_stickies;
	//! Pointer to the global array with the parameters
	stickyParameter *paramsArray;
    };
}

#endif
