// This file is part of the brownmove simulation package
// 
// V1.4  Aug 2012
// 
// (C) 2003 - 2012 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#include "globalstuff.h"

// using namespace brown;

namespace brown {
	// some global variables for global stuff
	// Q: Should we add the timestep here? => No, as long as it does not change during the simulation
	// 
	double KAPPA = 0.99;			// for physiological salt concentration of 0.09M
	double EPS_H2O = 78.0;
	double BETA = 1.0/(K_B*293.0);	// default temp = 20 degrees
	
	double HBOND_DEPTH = 25.0;		// average H-bond: 25 kJ/mol for the H-H-potential
	double HBOND_WIDTH = 0.05;		// H-H-potential width in nm
	double HBOND_CUTOFF = 1.5;		// ignore H bond if donor-H and acceptor-H are further apart
	double HBOND_SPRING = 30.0;		//  repulsive spring constant between donor and acceptor


	

	// Read next line, chop off leading whitespace and ignore comment lines
	int myGetLine(FILE *fp_in, char *newLine, const int maxLen, std::map<std::string, std::string> *constants, bool noLowerCase,
				  std::list<std::string> *repeatText, const bool readRepeat, const bool repeating, std::list<std::string>::iterator &repPtr)
	{
		int c;
		int count = 0;
		bool leadingWhite = true;
		bool ignoreLine = true;
		char myLine [2049];
		char endRepeat[] = "endrepeat";
		
		// std::cerr << "myGetLine: start reading" << std::endl;
		
		if(repeating == true)
		{
			// std::cerr << "myGetLine: repeating = true" << std::endl;
			strcpy(myLine, repPtr->c_str());
			repPtr++;
		}
		else 
		{
			bool readSilent = true;
			
			while(readSilent)
			{
				readSilent = false;		// activate switch
				ignoreLine = true;		// have to reset this one
				
				while(ignoreLine) 
				{
					leadingWhite = true;
					count = 0;
					
					c = fgetc(fp_in);
					
					// std::cerr << "myGetLine:    count = " << count << ",  c = " << (char)(c) << std::endl;
					
					while(c != '\n') {
						if(c == EOF) {
							return(c); 
						}
						
						if((c != ' ' && c != '\t')  || leadingWhite == false) {
							leadingWhite = false;
							myLine[count] = c;
							count++;
							if(count >= maxLen) {
								myLine[count-1] = '\0';
								// return(0);
								break;
							}
						}
						c = fgetc(fp_in);
						// std::cerr << "   myGetLine: count = " << count << ",  c = " << (char)(c) << std::endl;
					}
					
					myLine[count] = '\0';
					if(count > 0 && myLine[0] != '#') {
						ignoreLine = false; 
					}
				}
				
				// std::cerr << "myGetLine: read myLine = >>" << myLine << "<< from file" << std::endl;
				
				
				// save read line into repeat buffer
				if(readRepeat == true) {
					repeatText->push_back(std::string(myLine));
					
					// std::cerr << "myGetLine: repeatText-buffer:" << std::endl;
					// for(std::list<std::string>::iterator it = repeatText->begin(); it != repeatText->end(); it++) {
					// 	std::cerr << "   " << (*it) << std::endl;
					// }
					
					// check for "endrepeat"
					int i;
					for(i=0; i<9; i++) {
						if(tolower(myLine[i]) != endRepeat[i]) {
							// std::cerr << "myGetLine: comparing >>" << myLine << " to >>endRepeat<<  i = " << i << "  " << tolower(myLine[i]) << " - " << endRepeat[i] << std::endl;
							break;
						}
					}
					// std::cerr << "myGetLine: Comparison done, i = "<< i << std::endl;
					
					if(i<8) {
						// std::cerr << "  myGetLine: readSilent = true" << std::endl;
						readSilent = true;
					}
				}
			}
		}
		
		
		// std::cerr << "myGetLine: reading done, start processing: myLine = >>" << myLine << "<<" << std::endl;
		
		// check for replacements
		int i, k, it, itt;
		bool inConst = false;
		char target[2048];
		
		k = 0;
		i = 0;
		while(myLine[i] != '\0' || inConst)
		{
			char next = myLine[i];
			
			if(inConst)
			{
				// check whether a replacement is known
				if(constants->count(std::string(target))) {
					for(itt=0; itt<(*constants)[target].size(); itt++) {
						newLine[k] = (*constants)[target].c_str()[itt];
						k++;
					}
					inConst = false;
					i--;				// restart from current char
				}
				else  // keep reading target string
				{
					// std::cerr << "myGetLine: no replacement found for constant >>" << target << "<<" << std::endl;
					
					if(next != ' ' && next != '\t') {
						target[it] = next;
						target[it+1] = '\0';
						it++;
					} 
					else {	// replacement not known - exit!
						std::cerr << "myGetLine: constant >>" << target << "<< is undefined in line >>" << myLine << "<<" << std::endl;
						exit(-4);
					}
					// std::cerr << "myGetLine: target = >>" << target << "<<" << std::endl;
				}
			}
			
			else if(next == '$')  {	// start reading target
				inConst = true;
				it = 0;
				target[it] = '\0';
			} 
			else {				// just copy
				newLine[k] = next;
				k++;
			}
			
			i++;
		}
		newLine[k] = '\0';
		
		// std::cerr << "myGetLine: nearly finished, newLine = >>" << newLine << "<<" << std::endl;
		
		// convert 1st word to lower case
		i=0;
		while(newLine[i] != ' ' && newLine[i] != '\t') {
			if(!noLowerCase) {
				newLine[i] = tolower(newLine[i]);
			}
			i++;
		}
		// or convert second word to lower for bond types
		if(noLowerCase) {
			// std::cerr << "myGetLine: convert second word tolower(), i = " << i << "  newline[i] = >>" << newLine[i] <<"<<" << std::endl;
			
			while(newLine[i] == ' ' || newLine[i] == '\t') {
				i++;
			}
			while(newLine[i] != ' ' && newLine[i] != '\t') {
				newLine[i] = tolower(newLine[i]);
				i++;
			}
		}
		
		// std::cerr << "myGetLine: finished,       newLine = >>" << newLine << "<<" << std::endl;
		
		return(0);
	}
}


