// This file is part of the brownmove simulation package
// 
// V1.4  Aug 2012
// 
// (C) 2003 - 2012 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#include "pdbshape.h"

namespace brown{

	PDBShape::PDBShape(const PDBShape &other){
		PDBAtoms = other.PDBAtoms;
		actual_PDBAtoms = other.actual_PDBAtoms;
	}
	
	PDBShape::~PDBShape(){
		PDBAtoms.clear();
		actual_PDBAtoms.clear();
	}
	
	
	std::string PDBShape::describe(){
		ostringstream ausgabe("", ios::out);
		int i=0;
		
		ausgabe << "actual PDB atom positions";
		vector<PDBAtomPos>::iterator mlc;
		for(mlc = actual_PDBAtoms.begin(); mlc != actual_PDBAtoms.end(); ++mlc, i++){
			ausgabe << " : "<<(*mlc).pos[0]<<" "<<(*mlc).pos[1]<<" "<<(*mlc).pos[2] ; 
		}
		ausgabe <<" :";
		for(mlc = PDBAtoms.begin(); mlc != PDBAtoms.end(); ++mlc, i++){
			ausgabe << " : "<<(*mlc).pos[0]<<" "<<(*mlc).pos[1]<<" "<<(*mlc).pos[2] ; 
		}
		return(ausgabe.str());
	}
	
	
	void PDBShape::recalculate(const fullPosition &p) 
	{
		vector<PDBAtomPos>::iterator act = actual_PDBAtoms.begin();
		for(vector<PDBAtomPos>::iterator mlc = PDBAtoms.begin(); mlc != PDBAtoms.end(); ++mlc, ++act)
		{
			doublevec_set((*act).pos, (*mlc).pos);
			doublevec_rotate((*act).pos, p.rot);
			positionMove((*act).pos, p.pos);
			doublevec_set((*act).dr, (*act).pos);
			doublevec_sub((*act).dr, p.pos);
		}
	}
	
	
	void PDBShape::addPDBAtom(PDBAtomPos pdbp, const position &moleculeCenter){
		doublevec_set(pdbp.dr, pdbp.pos);
		doublevec_sub(pdbp.dr, moleculeCenter);
		PDBAtoms.push_back(pdbp);
		actual_PDBAtoms.push_back(pdbp);
	}
	
	
	
	void PDBShape::dumpStructure() {
		std::cout << "\n\t\tPDBShape myAtoms" << std::endl;
		
		for(std::vector<PDBAtomPos>::iterator it = PDBAtoms.begin(); it != PDBAtoms.end(); it++) {
			std::cout << "\t\t\t"<< (*it).atomIndex << "  " << (*it).atomType << "  " << (*it).resType <<"  "<< (*it).chain <<"  "<< (*it).resIndex <<"  "<< (*it).pos[0]<<" "<<(*it).pos[1]<<" "<<(*it).pos[2]<<"   "<<(*it).rawAtomType<< std::endl;
		}
		
		std::cout << "\t\tEnd" << std::endl;
	}

	
	int PDBShape::dumpPDBAtoms(long int index, std::string *raus) {
		int i = 1;
		ostringstream ausgabe("", ios::out);
		char zeile[256];
		
		// std::cerr << "PDBShape::dumpPDBAtoms: start..." << actual_PDBAtoms.size() <<  std::endl;
		// sprintf(zeile, "MODEL %8ld", index);
		// ausgabe << zeile << std::endl;
		
		for(std::vector<PDBAtomPos>::iterator it = actual_PDBAtoms.begin(); it != actual_PDBAtoms.end(); it++, i++) {
			if((*it).atomType == "TER") {
				sprintf(zeile, "TER %7d      %3s %c%4d",
						(*it).atomIndex, (*it).resType.c_str(), (*it).chain, (*it).resIndex);
			} 
			else {
				sprintf(zeile, "ATOM%7d  %3s %3s %c%4d   %8.3lf%8.3lf%8.3lf  1.00 20.00           %c",
						(*it).atomIndex, (*it).atomType.c_str(), (*it).resType.c_str(), (*it).chain, (*it).resIndex, (*it).pos[0]*10.0, (*it).pos[1]*10.0, (*it).pos[2]*10.0, (*it).rawAtomType);
			}
			
			ausgabe << zeile << std::endl;
		}
		
		// ausgabe << "ENDMDL" << std::endl;
		
		// std::cerr << "PDBShape::dumpPDBAtoms: done  ausgabe = " << ausgabe.str() << std::endl;

		(*raus) += ausgabe.str();
		return(i-1);
	}
	
}
