// This file is part of the brownmove simulation package
// 
// V1.4  Aug 2012
// 
// (C) 2003 - 2012 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "constants.h"
#include "types.h"
#include "vectormath.h"
#include "random.h"

#include "cup.h"
#include "wall.h"
#include "molecule.h"


#include "globalstuff.h"
#include "proteinparser.h"

using namespace brown;


void VMDHeader()
{
	std::cout << 
	"mol delete all\n" <<
	"mol load graphics gestalt\n" <<
	"mol load graphics vdw\n" <<
	"mol load graphics estat\n" <<
	"mol load graphics hbond\n" <<
	"mol load graphics hi\n" <<
	"mol load graphics bond\n" <<
	"mol load graphics vdw_label\n" <<
	"mol load graphics estat_label\n" <<
	"mol load graphics hbond_label\n" <<
	"mol load graphics hi_label\n" <<
	"mol load graphics bond_label\n" <<
	"mol load graphics timeline\n" <<
	"mol load graphics ramachandran\n" <<
	"graphics 6 materials off\n" <<
	"graphics 7 materials off\n" <<
	"graphics 8 materials off\n" <<
	"graphics 9 materials off\n" <<
	"graphics 10 materials off\n" <<
	"graphics 11 materials off\n" <<
	"graphics 12 materials off\n" <<
	"material add MSc1\n" <<
	"material change ambient MSc1 0.00\n" <<
	"material change specular MSc1 0.00\n" <<
	"material change diffuse MSc1 0.85\n" <<
	"material change shininess MSc1 0.53\n" <<
	"material change opacity MSc1 1.00\n" <<
	"graphics 0 material MSc1\n" <<
	"graphics 1 material MSc1\n" <<
	"graphics 2 material MSc1\n" <<
	"graphics 4 material MSc1\n" <<
	"graphics 3 material MSc1\n" <<
	"graphics 5 material MSc1\n" <<
	"graphics 0 color orange\n" <<
	"graphics 1 color yellow\n" <<
	"graphics 4 color silver\n" <<
	"graphics 5 color gray\n" <<
	"axes location off\n" <<
	"mol off 0\n" <<
	"mol off 3\n" <<
	"mol off 4\n" <<
	"mol off 10\n" <<
	"mol off 8\n" <<
	"mol off 6\n" <<
	"mol off 9\n" <<
	"mol off 7\n" <<
	"display projection orthographic\n" << std::endl;
}


void VMDBreak() {
	std::cout << "display resetview\n" <<
	"graphics 0 delete all\n" <<
	"graphics 1 delete all\n" <<
	"graphics 2 delete all\n" <<
	"graphics 3 delete all\n" <<
	"graphics 4 delete all\n" <<
	"graphics 5 delete all\n" <<
	"graphics 6 delete all\n" <<
	"graphics 7 delete all\n" <<
	"graphics 8 delete all\n" <<
	"graphics 9 delete all\n" <<
	"graphics 10 delete all\n" <<
	"graphics 11 delete all\n" <<
	"graphics 12 delete all\n" <<
	"graphics 6 materials off\n" <<
	"graphics 7 materials off\n" <<
	"graphics 8 materials off\n" <<
	"graphics 9 materials off\n" <<
	"graphics 10 materials off\n" <<
	"graphics 11 materials off\n" <<
	"graphics 12 materials off\n" <<
	"graphics 0 material MSc1\n" <<
	"graphics 1 material MSc1\n" <<
	"graphics 2 material MSc1\n" <<
	"graphics 4 material MSc1\n" <<
	"graphics 3 material MSc1\n" <<
	"graphics 5 material MSc1\n" <<
	"graphics 0 color orange\n" <<
	"graphics 1 color yellow\n" <<
	"graphics 4 color silver\n" <<
	"graphics 5 color gray\n" <<
	"mol off 0\n" <<
	"mol off 3\n" <<
	"mol off 4\n" <<
	"mol off 10\n" <<
	"mol off 8\n" <<
	"mol off 6\n" <<
	"mol off 9\n" <<
	"mol off 7\n\n" << std::endl;
}


void VMDFooter() {
	std::cout << "display resetview\n" << std::endl;
}



// void printHooks(std::vector<bondPos> &haken) 
// {
// 	std::vector<bondPos>::iterator it1, it2;
// 	// int i = 1;
// 	
// 	for(it1 = haken.begin(); it1 != haken.end(); ++it1) {
// 		for(it2 = it1; it2 != haken.end(); ++it2) {
// 			if((it1 != it2) && (*it1).idx == (*it2).idx) {
// 				std::cout << "graphics 5 cylinder {" <<10.0*(*it1).pos[0]<<" "<<10.0*(*it1).pos[1]<<" "<<10.0*(*it1).pos[2]<<"} {"<<10.0*(*it2).pos[0]<<" "<<10.0*(*it2).pos[1]<<" "<<10.0*(*it2).pos[2]<<"} radius 0.3 resolution 28" << std::endl;
// 			}
// 		}
// 	}
// }




int firstWords(const char *newLine, char *tag, char *tag2)
{
	int lang = strlen(newLine);
	int i = 0;

	if(sscanf(newLine, "%s %s", tag, tag2) == 2) {
		return(2); }
	else if(sscanf(newLine, "%s", tag) == 1) {
		tag2[0] = '\0';		// return empty string as second arg
		return(1); } 
	else {
		return(0);
	}
}


int main(int argc, char *argv[])
{
	enum EXTRACTTYPES { VDW, MARKERS, PDB } toExtract;
	
	if(argc <= 2) {
		cerr << "Extractor 1.4: read brownmove output from stdin and dump specified shapes of the given proteins" << endl;
		cerr << "   compiled" <<
		#include "compileInfo.h"
		<< endl;
		cerr << "\n  Usage:  " << argv[0] << "  <vdw|markers|pdb>  [\"-d <VariableName> <Value>\"] ... \"-p <proteinSetupFile> <proteinNumber(>= 1)>\"  [\"-p <proteinSetupFile> <proteinNumber(>= 1)>\"]  ... " << endl;
		return(-1);
	}
	
	// init parser and build template molecule
	double timeStep = 0.0;								// this is a dummy variable
	ProteinParser *p = new ProteinParser(timeStep);
	std::list<Molecule*> templateMols;

	// Map for textual replacements ("constants")
	std::map<std::string, std::string> constants;
	
	// a stack of FILE pointers and included filenames
	// list<FILE*> fileStack;		// no includes via the cmd line yet
	list<string> fileList;

	char molName[1024];
	char replaceString[1024];
	int molID, vdWMaxCol=100000, stickyMaxCol=100000;
	// define a single externalForce of type NONE => indicates to proteinParser to ignore all externalShape definitions
	// (actually, links them to a field of type NONE)
	list<externalForce> felder;
	externalForce xf;
	felder.push_back(xf);
	
	// which info to be dumped?
	if(strcmp(argv[1], "vdw") == 0) {
		toExtract = VDW;
	}
	else if(strcmp(argv[1], "markers") == 0) {
		toExtract = MARKERS;
	}
	else if(strcmp(argv[1], "pdb") == 0) {
		toExtract = PDB;
	}
	else {
		std::cerr << "extractor: unknown flag >>" << argv[1] << "<<" << std::endl;
		exit(-1);
	}
	
	// std::cerr << "extractor: argc = " << argc << std::endl;
	
	for(int itemp=2; itemp<argc; itemp++)
	{
		// std::cerr << "extractor: parsing arg >>" << argv[itemp] << "<<" << std::endl;
		// std::cerr << "extractor: itemp = " << itemp << ", argc = " << argc << std::endl;
		
		if(argv[itemp][0] == '-')
		{
			switch(argv[itemp][1])
			{
				// --- definition of a constant ---
				case 'd':
				// check whether there are another two args left, then treat them as constant and replacement
				if(argc <= itemp+2) {
					std::cerr << "extractor: not enough arguments left to define a constant (needs name and replacement string) ..." << std::endl;
					exit(-5);
				} 
				else if(argv[itemp+2][0] == '-') {
					std::cerr << "extractor: replacement string missing for >>" << argv[itemp+1] << "<<??? (may not start with \'-\' interactively)" << std::endl;
					exit(-5);
				}
				else {
					std::cerr << "extractor: constant >>" << argv[itemp+1] << "<< => >>" << argv[itemp+2] << "<<" << std::endl;
					constants[std::string(argv[itemp+1])] = std::string(argv[itemp+2]);
				}

				// increment itemp (I know, I shouldn't do that in a for loop...)
				itemp += 2;
				
				break;
				
				// ---- a protein ----
				case 'p':
				// check for number of args left
				if(argc > itemp+2) {
					if(sscanf(argv[itemp+2], "%d", &molID) != 1) {
						std::cerr << "extractor: >>" << argv[itemp+2] << "<< is not a valid index" << std::endl;
						exit(-5);
					}
					std::cerr << "extractor: reading molecule number " << molID << " from file >>" << argv[itemp+1] << "<<" << std::endl;
					
					Molecule *m1 = new Molecule("temp", 0);
					if(!(p->parse(m1, argv[itemp+1], (molID-1), vdWMaxCol, stickyMaxCol, felder, ANALYSIS, &constants, &fileList))) {
						std::cerr << "main::init_cup: error parsing molecule definition " << molID << " in file >>" << argv[itemp+1] << std::endl;
						exit(-1);
					}
					std::cerr << "building template molecule " << (m1->getid()).mclass << " done" << std::endl;
					templateMols.push_back(m1);
				}
				else {
					std::cerr << "extractor: not enough arguments left for a protein definition (needs filename + number)..." << std::endl;
					exit(-5);
				}
	
				itemp += 2;
				break;
				
				// ------
				default:
				std::cerr << "extractor: unknown flag >>" << argv[itemp][0] << argv[itemp][1] << "<<" << std::endl;
			}
			
		}
		else {
			std::cerr << "extractor: unidentifiable argument >>" << argv[itemp] << "<<" << std::endl;
		}
	}
	
	
	std::cerr << "Setup complete, start analyzing..." << std::endl;
	std::cerr << "  knowing " << templateMols.size() << " template molecules" << std::endl;
	
	char newLine[1024], tag1[1024], tag2[1024], protLabel[1024], gestLabel[1024], prefix[1024];
	double zeitPunkt, oldPunkt, dummy;
	long int idx, zeilen = 0, snapshots = 0;
	fullPosition fp;
	Gestalt *gestP;
	// std::vector<bondPos> haken;
	position CM;
	std::string ausgabe = "";
	
	// VMDHeader();
	
	// required args
	std::list<std::string> repeatText;
	std::list<std::string>::iterator repeatPointer = repeatText.begin();

	// loop: read stdin until EOF
	oldPunkt = -1.0;
	while(myGetLine(stdin, newLine, 1023, &constants, true, NULL, false, false, repeatPointer) != EOF)	// NO lowercase conversion!!!! (Labels!!)
	{
		// std::cerr << "Read: " << newLine << std::endl;
		
		// check for "timeline"
		firstWords(newLine, tag1, tag2);
		
		// std::cerr << "  tag1 = >>" << tag1 << "<<  tag2 = >>" << tag2 << "<<" << std::endl;
		
		if(strcmp(tag1, "timeline") == 0) {
			sscanf(tag2, "%lf", &zeitPunkt);
			
// 			// output of bonds
// 			if(haken.size() > 1) {
// 				printHooks(haken);
// 				VMDBreak();
// 			}
			
			// std::cerr << "\n\n  zeitPunkt = " << zeitPunkt << std::endl;
			// haken.clear();
			
			// print old timeline with updated number of entries
			if(oldPunkt >= 0.0) {
				// std::cout << "timeline " << oldPunkt << " " << zeilen << std::endl;
				snapshots++;
				if(toExtract == PDB) {
// 					if(snapshots > 1) {
// 						printf("ENDMDL\n");
// 					}
					printf("MODEL %8ld\n", snapshots);
				} else {
					printf("timeline %lf %ld\n", oldPunkt, zeilen);
				}
				std::cout << ausgabe;
				if(toExtract == PDB) {
					printf("ENDMDL\n");
				}
			}
				
			oldPunkt = zeitPunkt;
			zeilen = 0;
			ausgabe = "";
		}
		else
		{
			// extract position
			if(sscanf(newLine, "%s %ld %s %lf %lf %lf %lf %lf %lf %lf",
				   protLabel, &idx, gestLabel, &fp.pos[0], &fp.pos[1], &fp.pos[2], &fp.rot.phi, &fp.rot.n[0], &fp.rot.n[1], &fp.rot.n[2]) != 10)
			{
				std::cerr << "Extractor: error extracting position from line >>" << newLine << "<<" << std::endl;
				exit(-1);
			}
			
			// std::cerr << "  read position" << std::endl;
			
			std::list<Molecule*>::iterator iter;
			for(iter = templateMols.begin(); iter != templateMols.end(); ++iter)
			{
				// std::cerr << "  checking template protein " << ((*iter)->getid()).mclass << std::endl;
				
				if(string(protLabel) == ((*iter)->getid()).mclass)
				{
					// std::cerr << "Checking protein " << protLabel << ", Gestalt >>" << gestLabel << "<< at t = " << zeitPunkt << std::endl;

					// doublevec_set(CM, (*iter)->getCenterOfMass());
					
					// apply shift and rotation to gestalt of template molecule
					gestP = (*iter)->getGestalt(gestLabel);

					// std::cerr << "  identified Gestalt " << gestP->getLabel() << std::endl;
					
					gestP->recalculate(fp);
					
					// create identifier
					sprintf(prefix, "%s\t%ld\t%s_", protLabel, idx, gestLabel);
					// std::cerr << "  identifier = >>" << prefix << "<<" << std::endl;
					
					if(toExtract == VDW) {
						if(gestP->getvdWShape()) {
							zeilen += gestP->getvdWShape()->dumpSpheres(prefix, &ausgabe);
						}
					} 
					else if(toExtract == MARKERS) {
						if(gestP->getMarkerShape()) {
							zeilen += gestP->getMarkerShape()->dumpMarkers(prefix, &ausgabe);
						} 
						// else {
						// 	std::cerr << "  no MarkerShape" << std::endl;
						// }
					} 
					else if(toExtract == PDB) {
						if(gestP->getPDBShape()) {
							zeilen += gestP->getPDBShape()->dumpPDBAtoms(snapshots, &ausgabe);
						} 
					} 
	
					
					// gestP->getvdWShape()->dumpVMD(prefix);
					// gestP->getBondShape()->listHooks(haken);
					
					// std::cerr << "Extractor: haken.size() = " << haken.size() << std::endl;
				}
			}
		}
	}
	
	// print last timestep
	snapshots++;
	if(toExtract == PDB) {
// 		if(snapshots > 1) {
// 			printf("ENDMDL\n");
// 		}
		printf("MODEL %8ld\n", snapshots);
	} else {
		std::cout << "timeline " << oldPunkt << " " << zeilen << std::endl;
	}
	std::cout << ausgabe;
	if(toExtract == PDB) {
		printf("ENDMDL\n");
	}

	std::cerr << "Converted " << snapshots << " snapshots" << std::endl;
	
// 	// output of bonds
// 	if(haken.size() > 1) {
// 		printHooks(haken);
// 	}
// 	VMDFooter();
	
	// delete m1;
	
	return (0);
}
